@CHARSET "UTF-8";
.mastheadRight {
	display: table-cell;
	text-align: right;
	padding-right: 1.25em; 
	margin: 0px, 0px, 10px, 0px;
	border: solid green 0px;
	vertical-align: middle;
	width: 300px; 
	font-size: 0.625em; 
	font-family: 'Lucida Grande', verdana, helvetica, arial, sans-serif;
	font-weight: bold; 
}
.mastheadLeft {
	display: table-cell;
	padding-left: 0.75em;
	margin: 0px, 0px, 10px, 0px;
	word-spacing: 0.75em;
	border: solid green 0px;	
	vertical-align: middle;
	width: 430px;
 	text-align: left;	
	font-size: 0.625em; 
	font-family: 'Lucida Grande', verdana, helvetica, arial, sans-serif;
	font-weight: bold; 
}
.masthead {
	display: table;
	width: 100%; 
	top: 0px; 
	left: 0px; 
	position: absolute; 
	font-size: 16px;
	border: solid red 1px; 
	padding: 0px; 
	margin: 0px;
	vertical-align: middle;
	line-height: 1.5em;
}
.masthead .mastheadLeft .mastheadLogo {
	border: solid coral 0px;
	height: 16px;
	width: 16px;
	margin-right: 1em;
	vertical-align: middle;
}
.masthead .mastheadLeft ul {
	display: table-row;	
}
.masthead .mastheadRight ul{
	display: inline;	
	text-align: right;
}
.masthead .mastheadLeft ul li{
	display: table-cell;
	border: solid green 10px;
}
.masthead .mastheadRight ul li {
	display: inline;
	padding: 0px;
	margin: 0px;
}
.masthead .mastheadLeft ul, .masthead .mastheadRight ul {
	margin: 0px;
	padding-left: 0.5em;
	vertical-align: middle;
	list-style-type: none;
}
.masthead .mastheadLeft ul li, .masthead .mastheadRight ul li {
	border: 0px;
	margin: 0px;
	padding: 0px;
	list-style-type: none;
	vertical-align: middle;
}
.masthead .mastheadLeft ul li a, .masthead .mastheadRight ul li a {
	text-decoration: none;
	white-space: nowrap;
	vertical-align: middle;
}
.masthead .mastheadLeft ul li a:hover, .masthead .mastheadRight ul li a:hover {
	text-decoration: underline;
}
.masthead .mastheadLeft ul li a:active, .masthead .mastheadRight ul li a:active {
	text-decoration: underline;
}
.masthead #dbs a, .masthead #phedex a, .masthead #help a, .masthead #conddb a, .masthead #prodrequest a, .masthead #dashboard a{	
	margin: 0em 1em 0em 1em;
	padding: 1em 1em 0.7em 1em;
}
/* This next bit is for site dependant masthead */
.mastheadfooter {
	display: table-row;
	clear:both;
	position: relative;
	left:0px;
	width:100%;
	height:1.5em;
	padding:0em;
	margin:0px;
	border:dashed red 0px;
	line-height: 1.5em;
	font-size: 0.625em; 
	font-family: 'Lucida Grande', verdana, helvetica, arial, sans-serif;
	font-weight: bold;
}
.mastheadfooter .mhfootleft {
	display: table-cell;
	width:50%;
 	text-align: left;	
	border:solid blue 0px;
}
.mastheadfooter .mhfootright {
	display: table-cell;
	width:50%;
 	text-align: right;	
	border:solid blue 0px;
}
.mastheadfooter .mhfootleft ul {
	/*float: left;*/	
	display: inline;
	border:none blue 0px;
	margin:0px;
	padding-left: 2em;
	vertical-align:middle;
	list-style-type: none;
}
.mastheadfooter .mhfootright ul {
	display: inline;
	border:none blue 0px;
	margin:0px;
	padding-right: 2em;
	vertical-align:middle;
	list-style-type: none;
}
.mastheadfooter ul li {
	display: inline;
	border:none blue 0px;
	margin:0px;
	padding: 0em;
	list-style-type: none;
}
.mastheadfooter ul li a, .mastheadfooter ul li a:visited{
	text-decoration: none;
	border:none blue 0px;
	margin:0px;
	padding: 0em;
}
.mastheadfooter ul li a:hover{
	text-decoration: underline;
	border:none blue 0px;
}
.mastheadfooter ul li a:active{
	text-decoration: underline;
	border:none blue 0px;
}
